<?php

namespace App\Service;

use App\Exception;

class PaymentService {

    private $util;
    private $paymentFactory;

    public function __construct (\App\Util\Util $util,
                                 \App\Factory\PaymentFactory $paymentFactory) {
        $this->util = $util;
        $this->paymentFactory  = $paymentFactory;
    }
    
    public function getPayments($subsidiaryCode, $cdCaixa, $nrSeqVenda, $hasSubsidizedValue) {
        
        $arrayPayments = [];
        $payments = $this->paymentFactory->getPayments($subsidiaryCode, $cdCaixa , $nrSeqVenda);
        $index = 0;

        foreach($payments as $payment) {

            $paymentDetails = $this->paymentFactory->getPaymentDetails($subsidiaryCode, $cdCaixa , $nrSeqVenda, $payment['CDTIPORECE'], $payment['NRSEQUMOVI']);
            if(empty($paymentDetails))
                throw new Exception\ParamsException("Detalhes de forma de pagamento não encontrado. Verifique a parametrização de Financeiro.");
    
            if($paymentDetails['IDTIPORECE'] == "1" || $paymentDetails['IDTIPORECE'] == "2") {
                $cardType = $paymentDetails['IDTIPORECE'];
            }elseif($paymentDetails['IDTIPORECE'] == 'G' || $paymentDetails['IDTIPORECE'] == 'F') {
                $cardType = "3";
            }else{
                $cardType = null;
            }
    
            $receypt = [
                    'id'                 => str_pad($index +=  1, 3, '0', STR_PAD_LEFT),
                    'payment'            => $paymentDetails['NMTIPORECE'],
                    'code'               => $paymentDetails['CDTIPORECE'],
                    'externalCode'       => $paymentDetails['CDTPRECEINTE'],
                    'paymentType'        => $paymentDetails['IDTIPORECE'],
                    'cardType'           => isset($cardType) ? $cardType : '',
                    'cardDescription'    => isset($cardType) ? $this->getCardType($cardType) : '',
                    'movimentType'       => $paymentDetails['IDTIPOMOVIVE'],
                    'installmentPayment' => $this->util->tofloat($paymentDetails['QTPARCRECEB']),
                    'paymentSchedule'    => $this->util->tofloat($paymentDetails['NRDIASADMI']),
                    'Value'              => floatval($this->util->tofloat($paymentDetails['VRMOVIVEND'])),
                    'flagId'             => isset($paymentDetails['CDBANCARTSEFAZ']) ? $paymentDetails['CDBANCARTSEFAZ'] : '',
                    'flagDescription'    => isset($paymentDetails['CDBANCARTSEFAZ']) ? $this->getFlag($paymentDetails['CDBANCARTSEFAZ']) : '',
                    'nsu'                => isset($paymentDetails['CDNSUHOSTTEF']) ? $paymentDetails['CDNSUHOSTTEF'] : $paymentDetails['CDNSUTEF'],
                    'authorizationCode'  => isset($paymentDetails['NRAUTORIZATEF']) ? $paymentDetails['NRAUTORIZATEF'] : '',
                    'cnpjAdquirente'     => isset($paymentDetails['CDADMINCART']) ? $paymentDetails['CDADMINCART'] : '',
                    'nameAdquirente'     => isset($paymentDetails['CDADMINCART']) ? $paymentDetails['NMFANTCLIE'] : '',
                    'subsidizedPayment'  => $hasSubsidizedValue
                ];

            if(!empty($receypt))
                array_push($arrayPayments, $receypt);    
        }
        
        return $arrayPayments;
    }
    
    public function getCardType($cardType) {
        $types = array(
            '1' => 'Cartao Credito',
            '2' => 'Cartao Debito',
            '3' => 'Voucher'
        );
        
        return isset($types[$cardType]) ? $types[$cardType] : '';
        
    }
    
    public function getFlag($flag) {
        $types = array(
                '01' => 'Visa',
                '02' => 'Mastercard',
                '03' => 'American Express',
                '04' => 'Sorocred',
                '05' => 'Diners Club',
                '06' => 'Elo',
                '07' => 'Hipercard',
                '08' => 'Aura',
                '09' => 'Cabal',
                '10' => 'Alelo',
                '11' => 'Banes Card',
                '12' => 'CalCard ',
                '13' => 'Credz',
                '14' => 'Discover',
                '15' => 'GoodCard',
                '16' => 'GreenCard',
                '17' => 'Hiper',
                '18' => 'JcB',
                '19' => 'Mais',
                '20' => 'MaxVan',
                '21' => 'Policard',
                '22' => 'RedeCompras',
                '23' => 'Sodexo',
                '24' => 'ValeCard',
                '25' => 'Verocheque',
                '26' => 'VR',
                '27' => 'Ticket',
                '99' => 'Outros'
        );
        
        return isset($types[$flag]) ? $types[$flag] : '';
    }
    
    public function processPayment($payment) {

        if((isset($payment['code']) && !empty($payment['code'])) && empty($payment['externalCode'])) {
            $cdTipoRece = $this->paymentFactory->getCodePayment($payment['code']);
        }
        
        if(empty($cdTipoRece) && isset($payment['externalCode']) && !empty($payment['externalCode'])) {
            $cdTipoRece = $this->paymentFactory->getCodePayment($payment['externalCode']);
        }
        
        if(empty($cdTipoRece)) {
            throw new Exception\ParamsException(
                "O tipo de recebimento {$payment['payment']} não foi encontrado. Code: {$payment['code']} | externalCode: {$payment['externalCode']}."); 
        }
        
        return $cdTipoRece;
    }
}